/// アプリケーションの設定を操作するクラスの定義。
/**
	@file ini.d
**/

module config.ini;
private import config.config;
private import std.string;
private import std.file;

/// アプリケーションの設定を操作するクラス。
class ConfigIni
{
	this()
	{
		mEncoder = &dummyConverter;
		mDecoder = &dummyConverter;
	}
	
	/// 文字コード変換関数を指定して生成する。
	/**
		@param[in] aEncoder UTF8から任意の文字コードに変換する。
		@param[in] aDecoder 任意の文字コードからUTF8に変換する。
	**/
	this(char[] function(char[]) aEncoder, char[] function(char[]) aDecoder)
	{
		mEncoder = aEncoder;
		mDecoder = aDecoder;
	}
	
	/// セクションの存在を確認する。
	int isExistSection(char[] aName)
	{
		return ((aName in mSections) != null);
	}

	/// セクションを取得する。
	Section getSection(char[] aName)
	{
		assert(isExistSection(aName));
		return mSections[aName];
	}
	
	/// セクションを追加する。
	void addSection(Section aSection)
	{
		mSections[aSection.name] = aSection;
	}

//	void insertSection(Section aSection, int pos);

	/// セクションを削除する。
	void deleteSection(char[] aSectionName)
	{
		if(isExistSection(aSectionName))
			mSections.remove(aSectionName);
	}

	int sectionCount(){ return mSections.length; }
	
	/// ファイルから読み込む。
	bool loadFile(char[] aFileName)
	{
		char[] content = cast(char[])read(aFileName);
		loadText(content);
		return true;
	}

	/// ファイルに書き込む。
	bool saveFile(char[] aFileName)
	{
		if(exists(aFileName))
			remove(aFileName);
		foreach(Section sec; mSections)
		{
			// 保持している関数でエンコード
			sec.printIni(delegate void(char[] str){ append(aFileName, mEncoder(str)); });
			append(aFileName, "\r\n");
		}
		return true;
	}

	/// 文字列を解析して読み込む。
	int loadText(char[] aText)
	{
		clearAll();
		char[][] lines = splitlines(mDecoder(aText));
		char[] cur_sec_name;
		foreach(char[] line; lines)
		{
			// コメント、空白のみの場合は読み飛ばす。
			if(line.length == 0)
			{
				continue;
			}
			if(isComment(line))
			{
				continue;
			}
			else if(isSection(line))
			{
				char[] name = extractSectionName(line);
				if(name.length == 0)
					continue;
				Section sec = new Section(name);
				addSection(sec);
				cur_sec_name = name;
			}
			else
			{
				// セクションが無ければ無視
				if(cur_sec_name.length == 0 || !isExistSection(cur_sec_name))
				{
					continue;
				}
				char[] key = extractKey(line);
				// キーが無ければ無視
				if(key.length == 0)
				{
					continue;
				}
				mSections[cur_sec_name].setValue(key, extractValue(line));
			}
		}
		return (sectionCount() > 0);
	}

	/// コメントかどうか確認する。
	int isComment(char[] aLine)
	{
		char[] line = stripl(aLine);
		if(line.length == 0)
			return true;
		if(line[0] == ';')
			return true;
		return false;
	}
	
	/// セクションかどうか確認する。
	int isSection(char[] aLine)
	{
		char[] line = stripl(aLine);
		int s = find(line, '[');
		int e = rfind(line, ']');
		return (s != -1 && e != -1 && s < e);
	}

	/// セクション名を抽出する。
	char[] extractSectionName(char[] aLine)
	{
		char[] line = strip(aLine);
		int s = find(line, '[');
		int e = rfind(line, ']');
		assert(s != -1 && e != -1 && s < e);
		s++;
		char[] name = line[s..e];
		return name;
	}
	
	/// キーを抽出する。
	char[] extractKey(char[] aLine)
	{
		char[][] item = split(aLine, "=");
		if(item.length == 0)
			return "";
		return strip(item[0]);
	}
	
	/// 値を抽出する。
	char[] extractValue(char[] aLine)
	{
		char[][] item = split(aLine, "=");
		if(item.length == 0)
			return "";
		return strip(item[1]);
	}
	
	/// 全削除する。
	void clearAll()
	{
		char[][] names = mSections.keys;
		foreach(char[] name; names)
		{
			mSections.remove(name);
		}
	}

	/// ダミーの文字コード変換
	static char[] dummyConverter(char[] aStr)
	{
		return aStr;
	}

private:
	/// セクションを保持する。
	Section[char[]] mSections;
	/// UTF8からの文字エンコード
	char[] function(char[]) mEncoder;
	/// UTF8への文字デコード
	char[] function(char[]) mDecoder;
} 

unittest
{
	ConfigIni cfg = new ConfigIni;
	assert(cfg.isSection("[section]"));
	assert(cfg.isComment(";"));
	assert(cfg.isComment("	  ;"));
	// extract
	char[] sec_name = cfg.extractSectionName("[section]");
	Section sec = new Section(sec_name);
	char[] key = cfg.extractKey("key=value");
	assert(key == "key");
	char[] value = cfg.extractValue("key=value");
	assert(value == "value");
	// add
	sec.setValue(key, value);
	cfg.addSection(sec);
	assert(cfg.sectionCount() > 0);
	assert(cfg.isExistSection(sec_name));
	// find, get
	assert(cfg.getSection(sec_name).name == sec_name);
	assert(cfg.getSection(sec_name).getValueAsText(key) == value);
	// save load
	cfg.saveFile("config.ini");
	cfg.clearAll();
	assert(cfg.sectionCount == 0);
	cfg.loadText("[section]\r\nkey=value\r\n");
	assert(cfg.getSection("section").name == "section");
	assert(cfg.getSection("section").getValueAsText("key") == "value");
	cfg.clearAll();
	cfg.loadFile("config.ini");
	assert(cfg.getSection("section").name == "section");
	assert(cfg.getSection("section").getValueAsText("key") == "value");
}
