/// セクション
/**
	@file section.d
**/

module config.config;
private import std.string;
private import std.ctype;

/// セクションを示すクラス
class Section
{
	this(char[] aName)
	{
		mName = aName;
	}

	// setter
	
	/// 真偽値を設定する。
	/**
		キーが既にある場合は上書きする。
		trueの場合は"true"、falseの場合は"false"が設定される。
	**/
	void setValue(char[] aKey, bool aValue)
	{
		char[] value = aValue ? "true" : "false";
		setValue(aKey, value);
	}
	
	/// 整数値を設定する。
	/**
		キーが既にある場合は上書きする。
		許容桁数は64
	**/
	void setValue(char[] aKey, int aValue)
	{
		char[] value = std.string.toString(aValue);
		setValue(aKey, value);
	}

	/// 実数値を設定する。
	/**
		キーが既にある場合は上書きする。
		許容桁数は64
	**/
	void setValue(char[] aKey, float aValue)
	{
		char[] value = std.string.toString(aValue);
		setValue(aKey, value);
	}

	/// 文字列を設定する。
	/**
		キーが既にある場合は上書きする。
	**/
	void setValue(char[] aKey, char[] aValue)
	{
		mItems[aKey] = aValue;
	}
	
	// getter
	
	/// 真偽値として取得する。
	/**
		キーが無い場合はデフォルトとする。
		"true"以外の文字列は全てデフォルトとする。
	**/
	bool getValueAsBool(char[] aKey, bool aDefault=false)
	{
		if(!isExistKey(aKey))
		{
			return aDefault;
		}
		return mItems[aKey] == "true" ? true : false;
	}

	/// 整数値として取得する。
	/**
		キーが無い場合はデフォルトとする。
		非整数の文字列は全てデフォルトとする。
	**/
	int getValueAsInt(char[] aKey, int aDefault=0)
	{
		if(!isExistKey(aKey))
		{
			return aDefault;
		}
		debug{printf("%.*s, %d\n", mItems[aKey], mItems[aKey].length);}
		foreach(char c; mItems[aKey])
		{
			if(!(isdigit(c) || c == '-'))
				return aDefault;
		}
		return cast(int)atoi(mItems[aKey]);
	}

	/// 実数値として取得する。
	/**
		キーが無い場合はデフォルトとする。
		非実数の文字列は全てデフォルトとする。
	**/
	float getValueAsFloat(char[] aKey, float aDefault=0.0)
	{
		if(!isExistKey(aKey))
		{
			return aDefault;
		}
		foreach(char c; mItems[aKey])
		{
			if(!(isdigit(c) || c == '.' || c == '-'))
				return aDefault;
		}
		return atof(mItems[aKey]);
	}

	/// 文字列として取得する。
	/**
		キーが無い場合はデフォルトとする。
	**/
	char[] getValueAsText(char[] aKey, char[] aDefault="")
	{
		if(!isExistKey(aKey))
		{
			return aDefault;
		}
		return mItems[aKey];
	}
	
	/// キーがあるかどうかを確認する。
	int isExistKey(char[] aKey)
	{
		return ((aKey in mItems) != null);
	}

	/// キーを削除する。
	void removeKey(char[] aKey)
	{
		mItems.remove(aKey);
	}

	/// キーの数を確認する。
	int itemCount(){ return mItems.length; }

	/// INIファイル向けに出力する。
	/**
		セクションクラス内ではASCIIのみを扱うため、
		文字コードについては出力関数内にて対処すること。
	**/
	void printIni(void delegate(char[]) print, char[] aCrlf = "\x0d\x0a")
	{
		char[] str;
		str = "[" ~ mName ~ "]" ~ aCrlf;
		print(str);
		foreach(char[] key; mItems.keys)
		{
			str = key ~ "=" ~ mItems[key] ~ aCrlf;
			print(str);
		}
	}

	// property
	
	char[] name(){ return mName; }
	void name(char[] aName){ mName = aName; }

private:
	char[][char[]] mItems;
	char[] mName;
}

unittest
{
	char[] sec_name = "section";
	char[] sec_name2 = "section2";
	Section sec = new Section(sec_name);
	// property test
	assert(sec.name == sec_name);
	sec.name = sec_name2;
	assert(sec.name == sec_name2);
	// other test
	assert(sec.itemCount() == 0);
	assert(!sec.isExistKey("key"));

	// not key
	assert(sec.getValueAsBool("nokey") == false);
	assert(sec.getValueAsInt("nokey") == 0);
	assert(sec.getValueAsFloat("nokey") == 0.0);
	assert(sec.getValueAsText("nokey") == "");

	// bool valid
	sec.setValue("key", true);
	assert(sec.getValueAsBool("key") == true);
	assert(sec.getValueAsText("key") == "true");
	sec.setValue("key", false);
	assert(sec.getValueAsBool("key") == false);
	assert(sec.getValueAsText("key") == "false");
	// bool invalid
	assert(sec.getValueAsInt("key") == 0);
	assert(sec.getValueAsFloat("key") == 0.0);

	// int valid
	sec.setValue("key", 1);
//	printf("int test; %d\n", sec.getValueAsInt("key"));
	assert(sec.getValueAsInt("key") == 1);
	assert(sec.getValueAsFloat("key") == 1.0);
	assert(sec.getValueAsText("key") == "1");
	sec.setValue("key", -1);
	assert(sec.getValueAsInt("key") == -1);
	assert(sec.getValueAsFloat("key") == -1.0);
	assert(sec.getValueAsText("key") == "-1");
	// int invalid
	assert(sec.getValueAsBool("key") == false);
	
	// float valid
	sec.setValue("key", 1.5);
	assert(sec.getValueAsFloat("key") == 1.5);
	assert(sec.getValueAsText("key") == "1.5");
	sec.setValue("key", -1.5);
	assert(sec.getValueAsFloat("key") == -1.5);
	assert(sec.getValueAsText("key") == "-1.5");
	// float invalid
	assert(sec.getValueAsBool("key") == false);
	assert(sec.getValueAsInt("key") == 0);

	// string valid
	sec.setValue("key", "value");
	assert(sec.getValueAsText("key") == "value");
	// string invalid
	assert(sec.getValueAsBool("key") == false);
	assert(sec.getValueAsInt("key") == 0);
	assert(sec.getValueAsFloat("key") == 0.0);
	// print
	sec.setValue("key1", true);
	sec.setValue("key2", 100);
	sec.setValue("key3", 1234.1234);
	sec.setValue("key4", "string");
	
	sec.printIni(delegate void (char[] str){	printf(str);	});
}
