/// 各コマンド処理関数の定義。
/**
	@file	command.d
	@author	senzogawa
**/
module szpd.command;

private import std.string;
private import std.file;
private import szpd.result;
private import szpd.charcnv;

/// フォルダを作成する。
/**
	絶対パス、及び上位階層へアクセスするパスの指定はできない。
	また、日本語のフォルダは作成されない。
	@param[in]	aFolderName	フォルダ名
	@retval	1	フォルダ作成成功
	@retval	0	パスが不正
	@throw	FileException	mkdirの失敗
**/
Result makeDirectory(char[] aFolderName)
{
	Result result = checkLocalPath(aFolderName);
	if(result.num == Result.SUCCESS)
	{
		// 既にディレクトリがある場合はやめておく
		if(exists(aFolderName) && isdir(aFolderName))
		{
			result.set(Result.FAILURE, "失敗：既に同名のディレクトリがあります。");
		}
		else
		{
			mkdir(aFolderName);
		}
	}
	return result;
}


/// パスをチェックする。
/**
	絶対パス、及び上位階層へアクセスするパスの指定かどうか確認する。
**/
Result checkLocalPath(char[] aPath)
{
	// パスが無ければすぐに返す。
	if(aPath.length == 0)
	{
		return new Result(Result.ERROR, "エラー:パスがありません。");
	}
	// 絶対パスは不可
	// isabsを使ったら見事にすり抜けたので':'の検出で判定
	if(find(aPath, ":") != -1)
	{
		return new Result(Result.ERROR, "エラー:絶対パスは指定できません。");
	}
	// 上位階層は不可
	if(find(aPath, "..") != -1)
	{
		return new Result(Result.ERROR, "エラー:上位階層へのパスは指定できません。");
	}
	return new Result(Result.SUCCESS, "ok");
}

private import config.ini;
private import config.config;
/// INIファイルに設定する。
/**
	ファイルがあればファイルの設定箇所に書き込む。
	ファイルがなければ作成して書き込む。
**/
Result setIni(char[] aFileName, char[] aSectionName, char[] aKey, char[] aValue)
{
	Result result = checkLocalPath(aFileName);
	if(result.num == Result.SUCCESS)
	{
		ConfigIni cfg = new ConfigIni(&CharCnv.utf8ToSjis, &CharCnv.sjisToUtf8);
		// ファイルがあれば読み込んでおく。
		if(exists(aFileName))
		{
			// ファイルでなければエラーを返す。
			if(!isfile(aFileName))
			{
				result.set(Result.ERROR, "エラー：書き込めません。");
				return result;
			}
			cfg.loadFile(aFileName);
		}
		// セクションがあればそのまま設定。
		if(cfg.isExistSection(aSectionName))
		{
			cfg.getSection(aSectionName).setValue(aKey, aValue);
			cfg.saveFile(aFileName);
		}
		// セクションがなければ新規作成。
		else
		{
			Section sec = new Section(aSectionName);
			sec.setValue(aKey, aValue);
			cfg.addSection(sec);
			cfg.saveFile(aFileName);
		}
	}
	return result;
}


/// INIファイルから取得する。
/**
	NScripterネイティブのgetiniだとカレントディレクトリを見てくれないので一応用意。
	ファイルアクセスの同期などはとってないので、setIniの直後に呼ばない方がいい。
**/
Result getIni(char[] aFileName, char[] aSectionName, char[] aKey)
{
	Result result = checkLocalPath(aFileName);
	if(result.num == Result.SUCCESS)
	{
		ConfigIni cfg = new ConfigIni(&CharCnv.utf8ToSjis, &CharCnv.sjisToUtf8);
		// ファイルがなければ失敗を返す。
		if(!exists(aFileName))
		{
			result.set(Result.FAILURE, "失敗：ファイルがありません。");
		}
		else
		{
			// ファイルでなければエラーを返す。
			if(!isfile(aFileName))
			{
				result.set(Result.ERROR, "エラー：読み込めません。");
				return result;
			}
			cfg.loadFile(aFileName);
			result.set(Result.SUCCESS, cfg.getSection(aSectionName).getValueAsText(aKey));
		}
	}
	return result;
}


// 以下はデバッグ向けルーチン

debug
{

/// ログファイル名
const char[] LOG_FILE = "log.log";

/// ログを吐き出す。
/**
	ファイルは固定。ログはファイルの末尾に追加していく。
	その際、改行を自動で付加する。
	@param[in]	aLog	吐き出す内容
	@throw	FileException	appendの失敗
**/
Result putlog(char[] aLog)
{
	// UTF8に変換されているため、ShiftJISに戻す。
	// 多少冗長にはなるが、デバッグだけだからいいか。
	append(LOG_FILE, CharCnv.utf8ToSjis(aLog ~ "\x0d\x0a"));
	return new Result(Result.SUCCESS, "ok");
}

} // debug end
