/// プラグインのメインクラスの定義。
/**
	@file	plugin.d
	@author	senzogawa
**/
module szpd.plugin;

private import std.string;
private import szpd.charcnv;
private import szpd.result;

/// プラグインのメインクラス。
/**
	@class	Plugin	plugin.d
**/
class Plugin
{
	/// コンストラクタ。
	/**
		プラグインのexport関数の引数を渡せばいい。
	
		@param[in]	aParam	exec_dllに渡されるパラメータ
		@param[in]	aRetInt	返却用の数値ポインタ
		@param[in]	aRetStr	返却用の文字列ポインタ
	**/
	this(char* aParam, int* aRetInt, char* aRetStr)
	{
		char[] param = std.string.toString(aParam);
		mParam = CharCnv.sjisToUtf8(param);
		mRetInt = aRetInt;
		mRetStr = aRetStr;
		mRetStr[0] = '\0';
	}

	/// 返却する数値を設定する。
	/**
		@param[in]	aInt	返却する数値
	**/
	void setReturnValue(int aInt)
	{
		*mRetInt = aInt;
	}

	/// 返却する文字列を設定する。
	/**
		内部にて自動的にSJISに変換する。
		@param[in]	aStr	返却する文字列(UTF-8)
	**/
	void setReturnValue(char[] aStr)
	{
		char[] str = CharCnv.utf8ToSjis(aStr);
		int len = str.length;
		mRetStr[0..len] = str[0..len];
		mRetStr[len] = '\0';
	}

	/// 返却する数値と文字列を設定する。
	/**
		@param[in]	aInt	返却する数値
		@param[in]	aStr	返却する文字列
	**/
	void setReturnValue(int aInt, char[] aStr)
	{
		setReturnValue(aInt);
		setReturnValue(aStr);
	}

	/// 例外を設定する。
	/**
		例外の場合、toStringでShiftJIS文字列が返されるため、
		UTF8-SJIS変換は行わなずに設定する。
		@param[in]	aException	返却する例外
	**/
	void setException(Exception aException)
	{
		setReturnValue(Result.EXCEPTION);
		char[] text = CharCnv.utf8ToSjis("例外:") ~ aException.toString();
		mRetStr[0..text.length] = text[0..length];
		mRetStr[text.length] = '\0';
	}

	/// パラメータのコマンド部分を取得する。
	/**
		@return	コマンド部分の文字列
	**/
	char[] command()
	{
		int pos = find(mParam, ' ');
		if(pos != -1)
		{
			return mParam[0..pos];
		}
		else
		{
			return mParam[0..length];
		}
	}

	/// パラメータの引数部分を取得する。
	/**
		@return	引数部分の文字列
	**/
	char[] argument()
	{
		int pos = find(mParam, ' ');
		if(pos != -1)
		{
			pos++;
			return mParam[pos..length];
		}
		else
		{
			return "";
		}
	}

	/// パラメータがあるかどうかを確認する。
	/**
		@retval true	パラメータあり
		@retval false	パラメータなし
	**/
	bool isNoParam()
	{
		return (mParam.length == 0) ? true : false;
	}

private:
	/// 渡されたパラメータ
	char[] mParam;
	/// 返却する数値のポインタ
	int* mRetInt;
	/// 返却する文字列のバッファの先頭ポインタ
	char* mRetStr;
}
